import glob
import os
import warnings
from concurrent.futures import ThreadPoolExecutor
from pathlib import Path

COMPILER_ROOT = Path("/js")
COMPILER_DIR = COMPILER_ROOT / "compiler"
FASTLED_COMPILER_DIR = COMPILER_ROOT / "fastled/src/platforms/wasm/compiler"

HERE = Path(__file__).parent


def copy_task(src: str | Path) -> None:
    src = Path(src)
    if "entrypoint.sh" in str(src):
        return
    link_dst = COMPILER_ROOT / src.name

    # Handle shell scripts
    if src.suffix == ".sh":
        os.system(f"dos2unix {src} && chmod +x {src}")

    # if link exists, remove it
    if link_dst.exists():
        print(f"Removing existing link {link_dst}")
        try:
            os.remove(link_dst)
        except Exception as e:
            warnings.warn(f"Failed to remove {link_dst}: {e}")

    if not link_dst.exists():
        print(f"Linking {src} to {link_dst}")
        try:
            os.symlink(str(src), str(link_dst))
        except FileExistsError:
            print(f"Target {link_dst} already exists")
    else:
        print(f"Target {link_dst} already exists")


def make_links() -> None:
    # Define file patterns to include
    patterns = [
        "*.h",
        "*.hpp",
        "*.cpp",
        "*.py",
        "*.sh",
        "*.ino",
        "*.ini",
        "*.txt",
    ]

    # Get all matching files in compiler directory
    files = []
    for pattern in patterns:
        files.extend(glob.glob(str(COMPILER_DIR / pattern)))

    for pattern in patterns:
        files.extend(glob.glob(str(FASTLED_COMPILER_DIR / pattern)))

    # Process files in parallel using ThreadPoolExecutor
    with ThreadPoolExecutor(max_workers=16) as executor:
        executor.map(copy_task, files)


def init_runtime() -> None:
    os.chdir(str(HERE))
    make_links()


if __name__ == "__main__":
    init_runtime()



import argparse
import importlib.resources as pkg_resources
from dataclasses import dataclass
from multiprocessing import Process
from pathlib import Path

from fastled.server_fastapi_cli import run_fastapi_server_process
from fastled.server_flask import run_flask_server_process


def run_server_process(
    port: int, cwd: Path, certfile: Path | None = None, keyfile: Path | None = None
) -> Process:
    """Run the server in a separate process."""
    if True:
        # Use Flask server
        process = run_flask_server_process(
            port=port,
            cwd=cwd,
            certfile=certfile,
            keyfile=keyfile,
        )
    else:
        # Use FastAPI server
        process = run_fastapi_server_process(
            port=port,
            cwd=cwd,
            certfile=certfile,
            keyfile=keyfile,
        )
    return process


def get_asset_path(filename: str) -> Path | None:
    """Locate a file from the fastled.assets package resources."""
    try:
        resource = pkg_resources.files("fastled.assets").joinpath(filename)
        # Convert to Path for file-system access
        path = Path(str(resource))
        return path if path.exists() else None
    except (ModuleNotFoundError, AttributeError):
        return None


def start_process(
    path: Path,
    port: int,
    certfile: Path | None = None,
    keyfile: Path | None = None,
) -> Process:
    """Run the server, using package assets if explicit paths are not provided"""
    # Use package resources if no explicit path
    if certfile is None:
        certfile = get_asset_path("localhost.pem")
    if keyfile is None:
        keyfile = get_asset_path("localhost-key.pem")

    # _run_flask_server(path, port, certfile, keyfile)
    # run_fastapi_server_process(port=port, path=path, certfile=certfile, keyfile=keyfile)
    proc = run_server_process(port=port, cwd=path)
    # try:
    #     proc.join()
    # except KeyboardInterrupt:
    #     import _thread

    #     _thread.interrupt_main()
    return proc


@dataclass
class Args:
    fastled_js: Path
    port: int
    cert: Path | None
    key: Path | None


def parse_args() -> Args:
    parser = argparse.ArgumentParser(
        description="Open a browser to the fastled_js directory"
    )
    parser.add_argument(
        "fastled_js", type=Path, help="Path to the fastled_js directory"
    )
    parser.add_argument(
        "--port",
        "-p",
        type=int,
        default=5500,
        help="Port to run the server on (default: 5500)",
    )
    parser.add_argument(
        "--cert", type=Path, help="(Optional) Path to SSL certificate (PEM format)"
    )
    parser.add_argument(
        "--key", type=Path, help="(Optional) Path to SSL private key (PEM format)"
    )
    args = parser.parse_args()
    out: Args = Args(
        fastled_js=args.fastled_js,
        port=args.port,
        cert=args.cert,
        key=args.key,
    )
    if args.fastled_js is None:
        raise ValueError("fastled_js directory is required")
    return out


def main() -> None:
    args: Args = parse_args()
    fastled_js: Path = args.fastled_js
    port: int = args.port
    cert: Path | None = args.cert
    key: Path | None = args.key
    proc = start_process(
        path=fastled_js,
        port=port,
        certfile=cert,
        keyfile=key,
    )
    try:
        proc.join()
    except KeyboardInterrupt:
        import _thread

        _thread.interrupt_main()
        pass


if __name__ == "__main__":
    main()



import importlib.resources as pkg_resources
from dataclasses import dataclass
from pathlib import Path


@dataclass
class SslConfig:
    certfile: Path
    keyfile: Path


def get_asset_path(filename: str) -> Path | None:
    """Locate a file from the fastled.assets package resources."""
    try:
        resource = pkg_resources.files("fastled.assets").joinpath(filename)
        # Convert to Path for file-system access
        path = Path(str(resource))
        return path if path.exists() else None
    except (ModuleNotFoundError, AttributeError):
        return None


def get_ssl_config() -> SslConfig | None:
    """Get the keys for the server"""
    # certfile = get_asset_path("localhost-key.pem")
    # keyfile = get_asset_path("localhost.pem")
    # if certfile is None or keyfile is None:
    #     raise ValueError("Could not find keys for server")
    # # return certfile, keyfile
    # return SslConfig(certfile=certfile, keyfile=keyfile)
    return None



from multiprocessing import Process
from pathlib import Path

import uvicorn

# MAPPING = {
#     "js": "application/javascript",
#     "css": "text/css",
#     "wasm": "application/wasm",
#     "json": "application/json",
#     "png": "image/png",
#     "jpg": "image/jpeg",
#     "jpeg": "image/jpeg",
#     "gif": "image/gif",
#     "svg": "image/svg+xml",
#     "ico": "image/x-icon",
#     "html": "text/html",
# }


def _run_fastapi_server(
    port: int,
    cwd: Path,
    certfile: Path | None = None,
    keyfile: Path | None = None,
) -> None:
    # Uvicorn “reload” will watch your Python files for changes.
    import os

    os.chdir(cwd)
    uvicorn.run(
        "fastled.server_fastapi:app",
        host="127.0.0.1",
        port=port,
        reload=False,
        # reload_includes=["index.html"],
        ssl_certfile=certfile,
        ssl_keyfile=keyfile,
    )


def run_fastapi_server_process(
    port: int,
    cwd: Path | None = None,
    certfile: Path | None = None,
    keyfile: Path | None = None,
) -> Process:
    """Run the FastAPI server in a separate process."""
    cwd = cwd or Path(".")
    process = Process(
        target=_run_fastapi_server,
        args=(port, cwd, certfile, keyfile),
    )
    process.start()
    return process


if __name__ == "__main__":
    # Example usage
    proc = run_fastapi_server_process(port=8000)
    proc.join()



import argparse
from multiprocessing import Process
from pathlib import Path

from livereload import Server


def _run_flask_server(
    fastled_js: Path,
    port: int,
    certfile: Path | None = None,
    keyfile: Path | None = None,
) -> None:
    """Run Flask server with live reload in a subprocess

    Args:
        fastled_js: Path to the fastled_js directory
        port: Port to run the server on
        certfile: Path to the SSL certificate file
        keyfile: Path to the SSL key file
    """
    try:
        from flask import Flask, send_from_directory

        app = Flask(__name__)

        # Must be a full path or flask will fail to find the file.
        fastled_js = fastled_js.resolve()

        @app.route("/")
        def serve_index():
            return send_from_directory(fastled_js, "index.html")

        @app.route("/<path:path>")
        def serve_files(path):
            response = send_from_directory(fastled_js, path)
            # Some servers don't set the Content-Type header for a bunch of files.
            if path.endswith(".js"):
                response.headers["Content-Type"] = "application/javascript"
            if path.endswith(".css"):
                response.headers["Content-Type"] = "text/css"
            if path.endswith(".wasm"):
                response.headers["Content-Type"] = "application/wasm"
            if path.endswith(".json"):
                response.headers["Content-Type"] = "application/json"
            if path.endswith(".png"):
                response.headers["Content-Type"] = "image/png"
            if path.endswith(".jpg"):
                response.headers["Content-Type"] = "image/jpeg"
            if path.endswith(".jpeg"):
                response.headers["Content-Type"] = "image/jpeg"
            if path.endswith(".gif"):
                response.headers["Content-Type"] = "image/gif"
            if path.endswith(".svg"):
                response.headers["Content-Type"] = "image/svg+xml"
            if path.endswith(".ico"):
                response.headers["Content-Type"] = "image/x-icon"
            if path.endswith(".html"):
                response.headers["Content-Type"] = "text/html"

            # now also add headers to force no caching
            response.headers["Cache-Control"] = "no-cache, no-store, must-revalidate"
            response.headers["Pragma"] = "no-cache"
            response.headers["Expires"] = "0"
            return response

        server = Server(app.wsgi_app)
        # Watch index.html for changes
        server.watch(str(fastled_js / "index.html"))
        # server.watch(str(fastled_js / "index.js"))
        # server.watch(str(fastled_js / "index.css"))
        # Start the server
        server.serve(port=port, debug=True)
    except KeyboardInterrupt:
        import _thread

        _thread.interrupt_main()
    except Exception as e:
        print(f"Failed to run Flask server: {e}")
        import _thread

        _thread.interrupt_main()


def run(
    port: int, cwd: Path, certfile: Path | None = None, keyfile: Path | None = None
) -> None:
    """Run the Flask server."""
    try:
        _run_flask_server(cwd, port, certfile, keyfile)
        import warnings

        warnings.warn("Flask server has stopped")
    except KeyboardInterrupt:
        import _thread

        _thread.interrupt_main()
        pass


def parse_args() -> argparse.Namespace:
    """Parse the command line arguments."""
    parser = argparse.ArgumentParser(
        description="Open a browser to the fastled_js directory"
    )
    parser.add_argument(
        "fastled_js", type=Path, help="Path to the fastled_js directory"
    )
    parser.add_argument(
        "--port",
        "-p",
        type=int,
        required=True,
        help="Port to run the server on (default: %(default)s)",
    )
    parser.add_argument(
        "--certfile",
        type=Path,
        help="Path to the SSL certificate file for HTTPS",
    )
    parser.add_argument(
        "--keyfile",
        type=Path,
        help="Path to the SSL key file for HTTPS",
    )
    return parser.parse_args()


def run_flask_server_process(
    port: int,
    cwd: Path | None = None,
    certfile: Path | None = None,
    keyfile: Path | None = None,
) -> Process:
    """Run the Flask server in a separate process."""
    cwd = cwd or Path(".")
    process = Process(
        target=run,
        args=(port, cwd, certfile, keyfile),
    )
    process.start()
    return process


def main() -> None:
    """Main function."""
    args = parse_args()
    run(args.port, args.fastled_js, args.certfile, args.keyfile)


if __name__ == "__main__":
    main()



import subprocess
import sys
import time
from multiprocessing import Process
from pathlib import Path

from fastled.keyz import get_ssl_config

DEFAULT_PORT = 8089  # different than live version.
PYTHON_EXE = sys.executable
SSL_CONFIG = get_ssl_config()


# print(f"SSL Config: {SSL_CONFIG.certfile}, {SSL_CONFIG.keyfile}")


def _open_http_server_subprocess(
    fastled_js: Path,
    port: int,
) -> None:
    print("\n################################################################")
    print(f"# Opening browser to {fastled_js} on port {port}")
    print("################################################################\n")
    try:
        # Fallback to our Python server
        cmd = [
            PYTHON_EXE,
            "-m",
            "fastled.server_start",
            str(fastled_js),
            "--port",
            str(port),
        ]
        # Pass SSL flags if available
        if SSL_CONFIG and SSL_CONFIG.certfile and SSL_CONFIG.keyfile:
            cmd.extend(
                [
                    "--cert",
                    str(SSL_CONFIG.certfile),
                    "--key",
                    str(SSL_CONFIG.keyfile),
                ]
            )
            print(
                f"Running server on port {port} with certs: {SSL_CONFIG.certfile}, {SSL_CONFIG.keyfile}"
            )
        print(f"Running server on port {port}.")
        print(f"Command: {subprocess.list2cmdline(cmd)}")
        # Suppress output
        subprocess.run(
            cmd,
            stdout=subprocess.DEVNULL,
            # stderr=subprocess.DEVNULL,
        )  # type ignore
    except KeyboardInterrupt:
        print("Exiting from server...")
        sys.exit(0)


def is_port_free(port: int) -> bool:
    """Check if a port is free"""
    import httpx

    try:
        response = httpx.get(f"http://localhost:{port}", timeout=1)
        response.raise_for_status()
        return False
    except (httpx.HTTPError, httpx.ConnectError):
        return True


def find_free_port(start_port: int) -> int:
    """Find a free port starting at start_port"""
    for port in range(start_port, start_port + 100, 2):
        if is_port_free(port):
            print(f"Found free port: {port}")
            return port
        else:
            print(f"Port {port} is in use, finding next")
    raise ValueError("Could not find a free port")


def wait_for_server(port: int, timeout: int = 10) -> None:
    """Wait for the server to start."""
    from httpx import get

    future_time = time.time() + timeout
    while future_time > time.time():
        try:
            url = f"http://localhost:{port}"
            # print(f"Waiting for server to start at {url}")
            response = get(url, timeout=1)
            if response.status_code == 200:
                return
        except Exception:
            continue
    raise TimeoutError("Could not connect to server")


def open_browser_process(
    fastled_js: Path,
    port: int | None = None,
    open_browser: bool = True,
) -> Process:

    if port is not None and not is_port_free(port):
        raise ValueError(f"Port {port} was specified but in use")
    if port is None:
        port = find_free_port(DEFAULT_PORT)

    proc = Process(
        target=_open_http_server_subprocess,
        args=(fastled_js, port),
        daemon=True,
    )
    proc.start()
    wait_for_server(port)
    if open_browser:
        print(f"Opening browser to http://localhost:{port}")
        import webbrowser

        webbrowser.open(
            url=f"http://localhost:{port}",
            new=1,
            autoraise=True,
        )
    return proc


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(
        description="Open a browser to the fastled_js directory"
    )
    parser.add_argument(
        "fastled_js", type=Path, help="Path to the fastled_js directory"
    )
    parser.add_argument(
        "--port",
        type=int,
        default=DEFAULT_PORT,
        help=f"Port to run the server on (default: {DEFAULT_PORT})",
    )
    args = parser.parse_args()

    proc = open_browser_process(args.fastled_js, args.port, open_browser=True)
    proc.join()



from pathlib import Path

from fastapi import FastAPI, HTTPException
from fastapi.responses import FileResponse

# your existing MIME mapping, e.g.
MAPPING = {
    "js": "application/javascript",
    "css": "text/css",
    "wasm": "application/wasm",
    "json": "application/json",
    "png": "image/png",
    "jpg": "image/jpeg",
    "jpeg": "image/jpeg",
    "gif": "image/gif",
    "svg": "image/svg+xml",
    "ico": "image/x-icon",
    "html": "text/html",
}


"""Run FastAPI server with live reload or HTTPS depending on args."""
app = FastAPI(debug=True)
base = Path(".")


def no_cache_headers() -> dict[str, str]:
    return {
        "Cache-Control": "no-cache, no-store, must-revalidate",
        "Pragma": "no-cache",
        "Expires": "0",
    }


@app.get("/")
async def serve_index():
    index_path = base / "index.html"
    if not index_path.exists():
        raise HTTPException(status_code=404, detail="index.html not found")
    return FileResponse(
        index_path,
        media_type=MAPPING.get("html"),
        headers=no_cache_headers(),
    )


@app.get("/{path:path}")
async def serve_files(path: str):
    file_path = base / path
    if not file_path.exists() or not file_path.is_file():
        raise HTTPException(status_code=404, detail=f"{path} not found")

    ext = path.rsplit(".", 1)[-1].lower()
    media_type = MAPPING.get(ext)

    return FileResponse(
        file_path,
        media_type=media_type,
        headers=no_cache_headers(),
    )
